@@ -20,7 +20,6 @@ const gchar *STATUS_FILE_LOCATION = "/tmp/rauc-status.json";
  */
 const gchar *HTTP_HEADERS[] = {"upload_client_id", NULL};
 
-
 #define CGI_ERROR cgi_error_quark()
 
 static GQuark cgi_error_quark(void)
@@ -107,7 +106,7 @@ out:
 }
 
 /*
- * Last installation was successfull if LastError D-Bus property is an empty
+ * Last installation was successful if LastError D-Bus property is an empty
  * string.
  */
 static gboolean get_last_installation_success(RInstaller *installer, gboolean *last_install_success, GError **error)
@@ -246,7 +245,7 @@ static void try_cleanup_status_on_install_failure(RInstaller *installer)
 
 	/* Previous install failure is identified by "LastError" D-Bus property (also
 	 * set if no previous install happened), 100% install percentage of D-Bus
-	 * "Pogress" property and an existing status file (means the previous
+	 * "Progress" property and an existing status file (means the previous
 	 * installation was started via CGI).
 	 */
 	if (!last_install_success && install_percentage == 100 && g_file_test(STATUS_FILE_LOCATION, G_FILE_TEST_EXISTS)) {
@@ -674,12 +673,13 @@ static gint cgi_handler(int argc, char **argv)
 			goto error;
 		}
 	} else if (g_strcmp0("PUT", method) == 0) {
+		g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 		/* save stdin to temporary file */
 		if (!stdin_to_file(&error))
 			goto remove_bundle_on_error;
 
 		/* start rauc install */
-		if (!r_installer_call_install_sync(installer, BUNDLE_TARGET_LOCATION, NULL, &error))
+		if (!r_installer_call_install_bundle_sync(installer, BUNDLE_TARGET_LOCATION, g_variant_dict_end(&dict), NULL, &error))
 			goto remove_bundle_on_error;
 
 		print_headers("200 OK", "text/plain");
