@@ -39,13 +39,14 @@ typedef struct {
 	gchar *tls_ca; /* local file */
 	gboolean tls_no_verify;
 	GStrv headers; /* array of strings such as 'Foo: bar' */
-	GStrv info_headers; /* array of strings such as 'Foo: bar' */
+	GPtrArray *info_headers; /* array of strings such as 'Foo: bar' */
 
 	/* discovered information */
 	guint64 data_size; /* bundle size */
 	gchar *effective_url; /* url after redirects */
 	guint64 current_time; /* date header from server */
 	guint64 modified_time; /* last-modified header from server */
+	gchar *etag; /* etag received from the server */
 } RaucNBDServer;
 
 RaucNBDDevice *r_nbd_new_device(void);
@@ -65,7 +66,7 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(RaucNBDServer, r_nbd_free_server);
  *
  * @return TRUE on success, FALSE if an error occurred
  */
-gboolean r_nbd_setup_device(RaucNBDDevice *nbd_dev, GError **error);
+gboolean r_nbd_setup_device(RaucNBDDevice *nbd_dev, int *devicefd, GError **error);
 
 /**
  * Remove a previously configured NBD device from the kernel.
