@@ -45,6 +45,10 @@ G_DEFINE_AUTO_CLEANUP_FREE_FUNC(filedesc, close_preserve_errno, -1)
 
 static inline GSubprocess* r_subprocess_newv(GPtrArray *args, GSubprocessFlags flags, GError **error)
 {
+	g_return_val_if_fail(args, NULL);
+	g_return_val_if_fail(args->len, NULL);
+	g_return_val_if_fail(args->pdata[args->len-1] == NULL, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 	g_autofree gchar *call = g_strjoinv(" ", (gchar**) args->pdata);
 	g_log(R_LOG_DOMAIN_SUBPROCESS, G_LOG_LEVEL_DEBUG, "launching subprocess: %s", call);
 
@@ -53,6 +57,11 @@ static inline GSubprocess* r_subprocess_newv(GPtrArray *args, GSubprocessFlags f
 
 static inline GSubprocess * r_subprocess_launcher_spawnv(GSubprocessLauncher *launcher, GPtrArray *args, GError **error)
 {
+	g_return_val_if_fail(launcher, NULL);
+	g_return_val_if_fail(args, NULL);
+	g_return_val_if_fail(args->len, NULL);
+	g_return_val_if_fail(args->pdata[args->len-1] == NULL, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 	g_autofree gchar *call = g_strjoinv(" ", (gchar**) args->pdata);
 	g_log(R_LOG_DOMAIN_SUBPROCESS, G_LOG_LEVEL_DEBUG, "launching subprocess: %s", call);
 
@@ -412,6 +421,32 @@ void r_fakeroot_add_args(GPtrArray *args, const gchar *envpath);
 gboolean r_fakeroot_cleanup(const gchar *envpath, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
+/**
+ * Removes a temporary file and frees the filename string.
+ *
+ * Does nothing if the path is NULL. If the filename does not refer to a
+ * regular file, it only frees the string.
+ *
+ * @param filename path to the temporary file
+ */
+void r_tempfile_cleanup(gchar *filename);
+
+/* Use
+ *
+ *   g_auto(RTempFile) filename = g_build_filename(...);
+ *
+ * to declare a file that will be automatically removed when
+ * filename goes out of scope.
+ *
+ * If the file should become permanent on success, simply use
+ *
+ *   g_clear_pointer(&filename, g_free);
+ *
+ * to avoid the automatic cleanup.
+ */
+typedef gchar* RTempFile;
+G_DEFINE_AUTO_CLEANUP_FREE_FUNC(RTempFile, r_tempfile_cleanup, NULL)
+
 /**
  * Returns the contents of the GBytes as a '\0'-terminated string.
  *
@@ -449,3 +484,43 @@ gboolean r_semver_parse(const gchar *version_string, guint64 version_core[3], gc
  * @return TRUE if A<=B, FALSE otherwise
  */
 gboolean r_semver_less_equal(const gchar *version_string_a, const gchar *version_string_b, GError **error);
+
+/**
+ * Converts a duration given in seconds into a short human-readable string.
+ * The format is compact and space-separated, for example: "2h 15m 30s".
+ *
+ * Units are:
+ * - Days: "d"
+ * - Hours: "h"
+ * - Minutes: "m"
+ * - Seconds: "s"
+ *
+ * Units with zero values are omitted (except when the entire duration is zero,
+ * in which case "0s" is returned).
+ *
+ * @param total_seconds duration in seconds
+ *
+ * @return newly-allocated string representing the formatted duration
+ */
+gchar *r_format_duration(gint64 total_seconds);
+
+/**
+ * Compiles, matches and fetches the match in one call.
+ *
+ * This should only be used to simplify code in non-performance-critical places.
+ *
+ * @param pattern the regular expression
+ * @param string the string to search
+ *
+ * @return newly-allocated string with the matched substring or NULL
+ */
+gchar *r_regex_match_simple(const gchar *pattern, const gchar *string)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Reads CLOCK_BOOTTIME via clock_gettime().
+ *
+ * @return the time value in microseconds
+ */
+gint64 r_get_boottime(void)
+G_GNUC_WARN_UNUSED_RESULT;
