@@ -203,19 +203,7 @@ gboolean determine_slot_states(GError **error)
 	}
 
 	if (!booted) {
-		gboolean extboot = FALSE;
-
-		if (g_strcmp0(r_context()->bootslot, "/dev/nfs") == 0) {
-			g_message("Detected nfs boot, ignoring missing active slot");
-			extboot = TRUE;
-		}
-
 		if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
-			g_message("Detected explicit external boot, ignoring missing active slot");
-			extboot = TRUE;
-		}
-
-		if (extboot) {
 			/* mark all as inactive */
 			g_debug("Marking all slots as 'inactive'");
 			for (GList *l = slotlist; l != NULL; l = l->next) {
@@ -318,8 +306,10 @@ static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
 
 /* Selects a single appropriate inactive slot of root slot class
  *
- * Note: This function may be extended to be more sophisticated or follow a
- * certain policy for selecting an appropriate slot!
+ * If a global status file is used and multiple candidates are available, the
+ * one with the oldest 'installed' timestamp will be selected.
+ * If a slot with an unset timestamp is found, this one will be preferably
+ * instead.
  *
  * @param rootclass name of root slot class
  *
@@ -328,21 +318,70 @@ static gchar** get_all_manifest_slot_classes(const RaucManifest *manifest)
 static RaucSlot *select_inactive_slot_class_member(const gchar *rootclass)
 {
 	RaucSlot *iterslot;
+	RaucSlot *selectslot = NULL;
 	GHashTableIter iter;
 
 	g_return_val_if_fail(rootclass, NULL);
 
+	if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") == 0) {
+		g_debug("Selecting inactive slot for class '%s'. Strategy: 'first found'", rootclass);
+	} else {
+		g_debug("Selecting inactive slot for class '%s'. Strategy: 'oldest timestamp first')", rootclass);
+	}
+
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &iterslot)) {
 		if (iterslot->state != ST_INACTIVE)
 			continue;
 
-		if (g_strcmp0(iterslot->sclass, rootclass) == 0) {
+		if (g_strcmp0(iterslot->sclass, rootclass) != 0)
+			continue;
+
+		/* For per-slot status, we don't try to access the timestamp information
+		 * and just take the first appropriate slot */
+		if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") == 0) {
+			g_debug("Selected slot '%s'", iterslot->name);
 			return iterslot;
 		}
+
+		/* For the global slot status, context setup ensures the status is always initialized */
+		g_assert_nonnull(iterslot->status);
+
+		/* A slot that is "failed" should be updated preferably */
+		if (g_strcmp0(iterslot->status->status, "failed") == 0) {
+			g_debug("Slot '%s' has status 'failed'. Select it.", iterslot->name);
+			return iterslot;
+		}
+
+		/* A slot without a timestamp is always considered out-of-date and updatable */
+		if (!iterslot->status->installed_timestamp) {
+			g_debug("Slot '%s' has no valid timestamp. Consider outdated and select it.", iterslot->name);
+			return iterslot;
+		}
+
+		/* First slot found */
+		if (!selectslot) {
+			g_debug("Consider slot '%s' as first candidate.", iterslot->name);
+			selectslot = iterslot;
+			continue;
+		}
+
+		/* If currently checked slot is older than candidate, use as new candidate */
+		gint comp = g_date_time_compare(iterslot->status->installed_timestamp, selectslot->status->installed_timestamp);
+		if (comp < 0) {
+			g_autofree gchar *found = g_date_time_format(iterslot->status->installed_timestamp, RAUC_FORMAT_ISO_8601);
+			g_autofree gchar *current = g_date_time_format(selectslot->status->installed_timestamp, RAUC_FORMAT_ISO_8601);
+			g_debug("Slot '%s' has older timestamp (%s) than slot '%s' (%s). Consider as new candidate.", iterslot->name, found, selectslot->name, current);
+			selectslot = iterslot;
+		}
 	}
 
-	return NULL;
+	if (selectslot)
+		g_debug("Selected slot '%s'", selectslot->name);
+	else
+		g_debug("No slot selected");
+
+	return selectslot;
 }
 
 /* Map each slot class available to a potential target slot.
@@ -663,11 +702,12 @@ static gchar **add_system_environment(gchar **envp)
  *
  * @param update_source Path to the current bundle mount point
  * @param manifest Currently used manifest
+ * @param transaction Current transaction identifier
  * @param target_group Determined target group
  *
  * @return A newly allocated List of environment variables
  */
-static gchar **prepare_environment(gchar *update_source, RaucManifest *manifest, GHashTable *target_group)
+static gchar **prepare_environment(gchar *update_source, RaucManifest *manifest, gchar *transaction, GHashTable *target_group)
 {
 	GHashTableIter iter;
 	RaucSlot *slot;
@@ -682,6 +722,7 @@ static gchar **prepare_environment(gchar *update_source, RaucManifest *manifest,
 	envp = g_environ_setenv(envp, "RAUC_BUNDLE_MOUNT_POINT", update_source, TRUE);
 	/* Deprecated, included for backwards compatibility: */
 	envp = g_environ_setenv(envp, "RAUC_UPDATE_SOURCE", update_source, TRUE);
+	envp = g_environ_setenv(envp, "RAUC_TRANSACTION_ID", transaction, TRUE);
 
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
 	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
@@ -922,12 +963,11 @@ out:
 	return res;
 }
 
-static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bundledir, RaucManifest *manifest, GHashTable *target_group, GError **error)
+static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bundledir, RaucManifest *manifest, GHashTable *target_group, gchar **env, GError **error)
 {
 	GError *ierror = NULL;
 	g_autofree gchar* handler_name = NULL;
 	g_autoptr(GPtrArray) handler_args = NULL;
-	g_auto(GStrv) env = NULL;
 	gboolean res = FALSE;
 
 	r_context_begin_step_weighted("launch_and_wait_custom_handler", "Launching update handler", 0, 6);
@@ -954,7 +994,6 @@ static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bun
 	}
 	g_ptr_array_add(handler_args, NULL);
 
-	env = prepare_environment(bundledir, manifest, target_group);
 	res = launch_and_wait_handler(args, handler_name, (gchar**) handler_args->pdata, env, error);
 
 out:
@@ -1006,7 +1045,7 @@ static gboolean pre_install_checks(gchar* bundledir, GPtrArray *install_plans, G
 		}
 
 		if (!g_file_test(plan->image->filename, G_FILE_TEST_EXISTS)) {
-			if (plan->image->converted->len == 0) {
+			if (!plan->image->converted || plan->image->converted->len == 0) {
 				g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
 						"Source image '%s' not found in bundle", plan->image->filename);
 				return FALSE;
@@ -1065,12 +1104,8 @@ skip_filename_checks:
 
 static void update_slot_status(RaucSlotStatus *slot_state, const gchar* status, const RaucManifest *manifest, const RImageInstallPlan *plan, const RaucInstallArgs *args)
 {
-	g_autoptr(GDateTime) now = NULL;
-
 	r_slot_clear_status(slot_state);
 
-	now = g_date_time_new_now_utc();
-
 	slot_state->bundle_compatible = g_strdup(manifest->update_compatible);
 	slot_state->bundle_version = g_strdup(manifest->update_version);
 	slot_state->bundle_description = g_strdup(manifest->update_description);
@@ -1081,7 +1116,7 @@ static void update_slot_status(RaucSlotStatus *slot_state, const gchar* status,
 	slot_state->checksum.digest = g_strdup(plan->image->checksum.digest);
 	slot_state->checksum.size = plan->image->checksum.size;
 	slot_state->installed_txn = g_strdup(args->transaction);
-	slot_state->installed_timestamp = g_date_time_format(now, "%Y-%m-%dT%H:%M:%SZ");
+	slot_state->installed_timestamp = g_date_time_new_now_utc();
 	slot_state->installed_count++;
 }
 
@@ -1439,10 +1474,10 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 
 	if (boot_mark_slot) {
 		/* Mark boot slot non-bootable */
-		g_message("Marking target slot %s as non-bootable...", boot_mark_slot->name);
+		g_message("Marking target slot %s as non-bootable (bad)...", boot_mark_slot->name);
 		if (!r_mark_bad(boot_mark_slot, &ierror)) {
 			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MARK_NONBOOTABLE,
-					"Failed marking slot %s non-bootable: %s", boot_mark_slot->name, ierror->message);
+					"Failed marking slot %s non-bootable (bad): %s", boot_mark_slot->name, ierror->message);
 			g_clear_error(&ierror);
 			return FALSE;
 		}
@@ -1484,10 +1519,10 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	if (boot_mark_slot) {
 		if (r_context()->config->activate_installed) {
 			/* Mark boot slot bootable */
-			g_message("Marking target slot %s as bootable...", boot_mark_slot->name);
+			g_message("Marking target slot %s as bootable (active/primary)...", boot_mark_slot->name);
 			if (!r_mark_active(boot_mark_slot, &ierror)) {
 				g_propagate_prefixed_error(error, ierror,
-						"Failed marking slot %s bootable: ", boot_mark_slot->name);
+						"Failed marking slot %s bootable (active/primary): ", boot_mark_slot->name);
 				return FALSE;
 			}
 		} else {
@@ -1498,94 +1533,6 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	return TRUE;
 }
 
-static gchar* get_uptime(void)
-{
-	g_autofree gchar *contents = NULL;
-	g_autoptr(GError) ierror = NULL;
-	g_auto(GStrv) uptime = NULL;
-
-	if (!g_file_get_contents("/proc/uptime", &contents, NULL, &ierror)) {
-		g_warning("Failed to get uptime: %s", ierror->message);
-		return NULL;
-	}
-
-	/* file contains two values and a newline, 'chomp' in-place and split then */
-	uptime = g_strsplit(g_strchomp(contents), " ", 2);
-
-	return g_strdup(uptime[0]);
-}
-
-/* If the input key starts with RAUC_HTTP_, it returns a valid HTTP header
- * string with 'RAUC_HTTP_' replaced by 'RAUC-'.
- * If the input string does not start with RAUC_HTTP_, NULL is returned.
- */
-static gchar *system_info_to_header(const gchar *key, const gchar *value)
-{
-	g_autofree gchar *header_key = NULL;
-
-	g_return_val_if_fail(key, NULL);
-	g_return_val_if_fail(value, NULL);
-
-	if (!g_str_has_prefix(key, "RAUC_HTTP_"))
-		return NULL;
-
-	header_key = g_strdup(key + strlen("RAUC_HTTP_"));
-	for (size_t i = 0; i < strlen(header_key); i++) {
-		if (header_key[i] == '_')
-			header_key[i] = '-';
-	}
-
-	return g_strdup_printf("RAUC-%s: %s", header_key, value);
-}
-
-static gchar **assemble_info_headers(RaucInstallArgs *args)
-{
-	GPtrArray *headers = g_ptr_array_new_with_free_func(g_free);
-
-	g_return_val_if_fail(args, NULL);
-
-	if (!r_context()->config->enabled_headers)
-		goto no_std_headers;
-
-	for (gchar **header = r_context()->config->enabled_headers; *header; header++) {
-		/* Add static system information */
-		if (g_strcmp0(*header, "boot-id") == 0)
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Boot-ID: %s", r_context()->boot_id));
-		if (g_strcmp0(*header, "machine-id") == 0)
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Machine-ID: %s", r_context()->machine_id));
-		if (g_strcmp0(*header, "serial") == 0)
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Serial: %s", r_context()->system_serial));
-		if (g_strcmp0(*header, "variant") == 0)
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Variant: %s", r_context()->config->system_variant));
-		/* Add per-installation information */
-		if (g_strcmp0(*header, "transaction-id") == 0)
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Transaction-ID: %s", args->transaction));
-		/* Add live information */
-		if (g_strcmp0(*header, "uptime") == 0) {
-			g_autofree gchar *uptime = get_uptime();
-			g_ptr_array_add(headers, g_strdup_printf("RAUC-Uptime: %s", uptime));
-		}
-	}
-
-no_std_headers:
-
-	if (r_context()->system_info) {
-		GHashTableIter iter;
-		gchar *key = NULL;
-		gchar *value = NULL;
-
-		g_hash_table_iter_init(&iter, r_context()->system_info);
-		while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
-			gchar *header = system_info_to_header(key, value);
-			if (header)
-				g_ptr_array_add(headers, header);
-		}
-	}
-	g_ptr_array_add(headers, NULL);
-
-	return (gchar**) g_ptr_array_free(headers, FALSE);
-}
-
 gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 {
 	const gchar* bundlefile = args->name;
@@ -1617,7 +1564,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	// TODO: mount info in context ?
 	install_args_update(args, "Checking and mounting bundle...");
 
-	args->access_args.http_info_headers = assemble_info_headers(args);
+	args->access_args.http_info_headers = assemble_info_headers(args->transaction);
 
 	res = check_bundle(bundlefile, &bundle, CHECK_BUNDLE_DEFAULT, &args->access_args, &ierror);
 	if (!res) {
@@ -1625,6 +1572,21 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 		goto out;
 	}
 
+	if (args->require_manifest_hash) {
+		if (!bundle->manifest) {
+			/* only plain bundles have no manifest at this point */
+			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED, "Refusing to install plain bundle when using require-manifest-hash");
+			res = FALSE;
+			goto out;
+		}
+		if (g_strcmp0(args->require_manifest_hash, bundle->manifest->hash) != 0) {
+			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED, "Refusing to install bundle with unexpected hash (expected %s, got %s)",
+					args->require_manifest_hash, bundle->manifest->hash);
+			res = FALSE;
+			goto out;
+		}
+	}
+
 	if (bundle->manifest && bundle->manifest->bundle_format == R_MANIFEST_FORMAT_CRYPT && !bundle->was_encrypted) {
 		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED, "Refusing to install unencrypted crypt bundles");
 		res = FALSE;
@@ -1649,8 +1611,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 		goto umount;
 	}
 
-	handler_env = prepare_environment(bundle->mount_point, bundle->manifest, target_group);
-	handler_env = g_environ_setenv(handler_env, "RAUC_TRANSACTION_ID", args->transaction, TRUE);
+	handler_env = prepare_environment(bundle->mount_point, bundle->manifest, args->transaction, target_group);
 
 	if (r_context()->config->preinstall_handler) {
 		g_message("Starting pre install handler: %s", r_context()->config->preinstall_handler);
@@ -1693,7 +1654,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 
 	if (bundle->manifest->handler_name) {
 		g_message("Using custom handler: %s", bundle->manifest->handler_name);
-		res = launch_and_wait_custom_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
+		res = launch_and_wait_custom_handler(args, bundle->mount_point, bundle->manifest, target_group, handler_env, &ierror);
 	} else {
 		g_debug("Using default installation handler");
 		res = launch_and_wait_default_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
@@ -1789,25 +1750,33 @@ void install_args_free(RaucInstallArgs *args)
 	g_mutex_clear(&args->status_mutex);
 	g_assert_cmpint(args->status_result, >=, 0);
 	g_assert_true(g_queue_is_empty(&args->status_messages));
+	g_free(args->require_manifest_hash);
 	clear_bundle_access_args(&args->access_args);
 	g_free(args);
 }
 
-gboolean install_run(RaucInstallArgs *args)
+void install_run(RaucInstallArgs *args)
 {
-	g_autoptr(GThread) thread = NULL;
 	r_context_set_busy(TRUE);
 
 	g_message("Active slot bootname: '%s'", r_context()->bootslot);
 
-	thread = g_thread_new("installer", install_thread, args);
-	if (thread == NULL)
-		return FALSE;
-
-	return TRUE;
+	/* g_thread_new aborts if the thread cannot be created. */
+	GThread *thread = g_thread_new("installer", install_thread, args);
+	/* we don't need to keep a reference to this thread */
+	g_thread_unref(thread);
 }
 
-static const gchar *supported_http_headers[] = {"boot-id", "transaction-id", "machine-id", "serial", "variant", "uptime", NULL};
+static const gchar *supported_http_headers[] = {
+	"boot-id",
+	"transaction-id",
+	"machine-id",
+	"system-version",
+	"serial",
+	"variant",
+	"uptime",
+	NULL
+};
 
 gboolean r_install_is_supported_http_header(const gchar *header)
 {
