@@ -12,6 +12,7 @@ static GList* get_slots_by_identifier(const gchar *identifier, GError **error)
 	GList *slots = NULL;
 	RaucSlot *booted = NULL;
 
+	g_return_val_if_fail(identifier != NULL, NULL);
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
 	g_hash_table_iter_init(&iter, r_context()->config->slots);
@@ -133,7 +134,6 @@ gboolean r_mark_active(RaucSlot *slot, GError **error)
 {
 	RaucSlotStatus *slot_state;
 	GError *ierror = NULL;
-	g_autoptr(GDateTime) now = NULL;
 
 	g_return_val_if_fail(slot, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
@@ -150,9 +150,9 @@ gboolean r_mark_active(RaucSlot *slot, GError **error)
 
 	r_event_log_mark_active(slot);
 
-	g_free(slot_state->activated_timestamp);
-	now = g_date_time_new_now_utc();
-	slot_state->activated_timestamp = g_date_time_format(now, "%Y-%m-%dT%H:%M:%SZ");
+	if (slot_state->activated_timestamp)
+		g_date_time_unref(slot_state->activated_timestamp);
+	slot_state->activated_timestamp = g_date_time_new_now_utc();
 	slot_state->activated_count++;
 
 	if (!r_slot_status_save(slot, &ierror)) {
@@ -209,8 +209,10 @@ gboolean mark_run(const gchar *state,
 	g_autoptr(GList) slots = NULL;
 	g_autoptr(GError) ierror = NULL;
 
-	g_assert(slot_name == NULL || *slot_name == NULL);
-	g_assert(message != NULL && *message == NULL);
+	g_return_val_if_fail(state != NULL, FALSE);
+	g_return_val_if_fail(slot_identifier != NULL, FALSE);
+	g_return_val_if_fail(slot_name == NULL || *slot_name == NULL, FALSE);
+	g_return_val_if_fail(message != NULL && *message == NULL, FALSE);
 
 	slots = get_slots_by_identifier(slot_identifier, &ierror);
 	if (ierror) {
