@@ -10,6 +10,7 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+#include <time.h>
 
 #include "utils.h"
 
@@ -881,6 +882,19 @@ gboolean r_fakeroot_cleanup(const gchar *envpath, GError **error)
 	return rm_tree(tmpdir, error);
 }
 
+void r_tempfile_cleanup(gchar *filename)
+{
+	if (!filename)
+		return;
+
+	if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
+		if (g_unlink(filename) != 0)
+			g_warning("failed to remove %s", filename);
+	}
+
+	g_free(filename);
+}
+
 gchar *r_bytes_unref_to_string(GBytes **bytes)
 {
 	g_return_val_if_fail(bytes != NULL && *bytes != NULL, NULL);
@@ -1077,3 +1091,49 @@ gboolean r_semver_less_equal(const gchar *version_string_a, const gchar *version
 	else
 		return (pre_fields_a[i] == NULL) && (pre_fields_b[i] != NULL);
 }
+
+gchar *r_format_duration(gint64 total_seconds)
+{
+	gint64 seconds = total_seconds % 60;
+	gint64 minutes = (total_seconds / 60) % 60;
+	gint64 hours = (total_seconds / 3600) % 24;
+	gint64 days = total_seconds / (3600 * 24);
+
+	GString *result = g_string_new(NULL);
+
+	if (days)
+		g_string_append_printf(result, "%"G_GINT64_FORMAT "d ", days);
+	if (hours)
+		g_string_append_printf(result, "%"G_GINT64_FORMAT "h ", hours);
+	if (minutes)
+		g_string_append_printf(result, "%"G_GINT64_FORMAT "m ", minutes);
+	if (seconds || !total_seconds)
+		g_string_append_printf(result, "%"G_GINT64_FORMAT "s", seconds);
+
+	return g_strchomp(g_string_free(result, FALSE));
+}
+
+gchar *r_regex_match_simple(const gchar *pattern, const gchar *string)
+{
+	g_autoptr(GRegex) regex = NULL;
+	g_autoptr(GMatchInfo) match = NULL;
+
+	g_return_val_if_fail(pattern, NULL);
+	g_return_val_if_fail(string, NULL);
+
+	regex = g_regex_new(pattern, 0, 0, NULL);
+	if (g_regex_match(regex, string, 0, &match))
+		return g_match_info_fetch(match, 1);
+
+	return NULL;
+}
+
+gint64 r_get_boottime(void)
+{
+	struct timespec ts;
+
+	if (clock_gettime(CLOCK_BOOTTIME, &ts) != 0)
+		g_error("RAUC needs CLOCK_BOOTTIME (failed with %s)", g_strerror(errno));
+
+	return (((gint64) ts.tv_sec) * 1000000) + (ts.tv_nsec / 1000);
+}
