@@ -217,14 +217,6 @@ mountprefix=/mnt/myrauc/\n\
 [unknown]\n\
 foo=bar\n\
 ";
-	const gchar *unknown_key_cfg_file = "\
-[system]\n\
-compatible=FooCorp Super BarBazzer\n\
-bootloader=barebox\n\
-mountprefix=/mnt/myrauc/\n\
-foo=bar\n\
-";
-
 	pathname = write_tmp_file(fixture->tmpdir, "unknown_group.conf", unknown_group_cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -233,9 +225,16 @@ foo=bar\n\
 	g_assert_cmpstr(ierror->message, ==, "Invalid group '[unknown]'");
 	g_clear_error(&ierror);
 
-	g_free(pathname);
+	g_clear_pointer(&pathname, g_free);
 	g_clear_pointer(&config, free_config);
 
+	const gchar *unknown_key_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+mountprefix=/mnt/myrauc/\n\
+foo=bar\n\
+";
 	pathname = write_tmp_file(fixture->tmpdir, "unknown_key.conf", unknown_key_cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
@@ -706,6 +705,30 @@ bootname=the\tslot\n";
 	g_clear_error(&ierror);
 }
 
+static void config_file_boot_emmc_with_bootpart(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	g_autoptr(RaucConfig) config = NULL;
+	GError *ierror = NULL;
+	g_autofree gchar* pathname = NULL;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/mmcblk0boot0\n\
+type=boot-emmc\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "boot-emmc-bootpart.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_INVALID_DEVICE);
+	g_assert_cmpstr(ierror->message, ==, "slot.rootfs.0: 'device' must refer to the eMMC base device, not the boot partition");
+	g_clear_error(&ierror);
+}
+
 static void config_file_no_max_bundle_download_size(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
@@ -1084,7 +1107,7 @@ static void config_file_keyring_checks(ConfigFileFixture *fixture,
 	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	g_autofree gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *simple_cfg_file = "\
 [system]\n\
@@ -1100,7 +1123,8 @@ bootloader=barebox\n\
 path=/dev/null\n\
 allow-partial-chain=true\n\
 check-crl=true\n\
-check-purpose=codesign\n";
+check-purpose=codesign\n\
+allowed-signer-cns=SomeAllowedCN;OtherAllowedCN\n";
 
 	pathname = write_tmp_file(fixture->tmpdir, "simple.conf", simple_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1126,6 +1150,9 @@ check-purpose=codesign\n";
 	g_assert_true(config->keyring_allow_partial_chain);
 	g_assert_true(config->keyring_check_crl);
 	g_assert_cmpstr(config->keyring_check_purpose, ==, "codesign-rauc");
+	g_assert_cmpint(g_strv_length(config->keyring_allowed_signer_cns), ==, 2);
+	g_assert_cmpstr(config->keyring_allowed_signer_cns[0], ==, "SomeAllowedCN");
+	g_assert_cmpstr(config->keyring_allowed_signer_cns[1], ==, "OtherAllowedCN");
 }
 
 static void config_file_bundle_formats(ConfigFileFixture *fixture,
@@ -1134,7 +1161,7 @@ static void config_file_bundle_formats(ConfigFileFixture *fixture,
 	g_autoptr(RaucConfig) config = NULL;
 	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
-	g_autofree gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *default_cfg_file = "\
 [system]\n\
@@ -1652,6 +1679,9 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/bootname-tab", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_bootname_tab,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/boot-emmc-with-bootpart", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_boot_emmc_with_bootpart,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/no-max-bundle-download-size", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_no_max_bundle_download_size,
 			config_file_fixture_tear_down);
