@@ -150,6 +150,11 @@ static void event_log_test_log_write_json(EventLogFixture *fixture,
 	};
 	g_autofree gchar *contents = NULL;
 
+	if (!ENABLE_JSON) {
+		g_test_skip("Test requires RAUC being configured with \"-Djson=enabled\".");
+		return;
+	}
+
 	logger = g_new0(REventLogger, 1);
 	logger->name = g_strdup("testlogger");
 	logger->format = R_EVENT_LOGFMT_JSON;
@@ -301,7 +306,6 @@ static void event_log_test_structured_logging(EventLogFixture *fixture,
 	g_assert_nonnull(strstr(contents, "This is a test (mark) log message"));
 }
 
-
 /* Test setting up a logger for structured logging and log with r_event_log_message */
 static void event_log_test_log_utility(EventLogFixture *fixture,
 		gconstpointer user_data)
@@ -359,7 +363,6 @@ static void event_log_test_log_filtering(EventLogFixture *fixture,
 	g_assert_nonnull(strstr(contents, "Example second mark message"));
 }
 
-
 int main(int argc, char *argv[])
 {
 	RotationTestConfig *rot_test_conf;
